/*
 * Author: Richard Lord
 * Copyright (c) Richard Lord 2010
 * http://www.richardlord.net
 * 
 * Licence Agreement
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package net.richardlord.smartcomponents.twitter.buttons
{
	import flash.events.Event;
	import flash.events.IEventDispatcher;
	import flash.events.MouseEvent;
	
	import mx.binding.utils.ChangeWatcher;
	import mx.controls.Button;
	
	import net.richardlord.smartcomponents.twitter.data.User;
	import net.richardlord.smartcomponents.twitter.events.AuthenticateEvent;
	
	public class LoginLogoutButton extends Button
	{
		[Inspectable( environment="none" )]
		[Dispatcher]
		public var eventBus : IEventDispatcher;
		
		[Inspectable( environment="none" )]
		[Inject]
		public var user : User;
		
		[Inspectable( defaultValue="Login", category="Common" )]
		public var loginLabel : String;
		
		[Inspectable( defaultValue="Logout", category="Common" )]
		public var logoutLabel : String;
		
		private var userWatcher : ChangeWatcher;
		
		public function LoginLogoutButton()
		{
			buttonMode = true;
			addEventListener( MouseEvent.CLICK, click, false, 0, true );
		}
		
		[PostConstruct]
		public function setUp() : void
		{
			userWatcher = ChangeWatcher.watch( user, "token", setLoginLogout );
			setLoginLogout();
		}
		
		private function setLoginLogout( event : Event = null ) : void
		{
			if ( user.token == null )
			{
				label = loginLabel;
			}
			else
			{
				label = logoutLabel;
			}
		}
		
		public function click( event : Event = null ) : void
		{
			if ( user.token == null )
			{
				eventBus.dispatchEvent( new AuthenticateEvent( AuthenticateEvent.TOGGLE_AUTHENTICATE ) );
			}
			else
			{
				user.token = null;
			}
		}
	}
}
