/*
 * Author: Richard Lord
 * Copyright (c) Richard Lord 2010
 * http://www.richardlord.net
 * 
 * Licence Agreement
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package net.richardlord.smartcomponents.twitter.services.parsers
{
	import mx.formatters.DateBase;
	
	import net.richardlord.smartcomponents.twitter.data.Tweet;
	
	public class TweetXmlParser
	{
		public function parseTweet( xml : XML ) : Tweet
		{
			var tweet : Tweet = new Tweet();
			tweet.id = parseFloat( xml.id.text() );
			tweet.created = parseDate( xml.created_at.text() );
			tweet.text = xml.text.text();
			tweet.html = xml.text.text();
			tweet.image = xml.user.profile_image_url.text();
			tweet.source = xml.source.text();
			tweet.authorName = xml.user.name.text();
			tweet.authorUri = xml.user.url.text();
			return tweet;
		}
		
		private function parseDate( time : String ) : Date
		{
			var year : int = parseInt( time.substr( -4, 4 ) );
			var monthString : String = time.substr( 4, 3 );
			var month : int = DateBase.monthNamesShort.indexOf( monthString );
			if ( month == -1 )
			{
				return new Date();
			}
			var day : int = parseInt( time.substr( 8, 2 ) );
			var hour : int = parseInt( time.substr( 11, 2 ) );
			var minute : int = parseInt( time.substr( 14, 2 ) );
			var second : int = parseInt( time.substr( 17, 2 ) );
			if ( time.substr( 20, 1 ) == "-" )
			{
				hour += parseInt( time.substr( 21, 2 ) );
				minute += parseInt( time.substr( 23, 2 ) );
			}
			else if ( time.substr( 20, 1 ) == "+" )
			{
				hour -= parseInt( time.substr( 21, 2 ) );
				minute -= parseInt( time.substr( 23, 2 ) );
			}
			return new Date( Date.UTC( year, month, day, hour, minute, second ) );
		}
	
	}
}