/*
 * Author: Richard Lord
 * Copyright (c) Richard Lord 2010
 * http://www.richardlord.net
 * 
 * Licence Agreement
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package net.richardlord.smartcomponents.twitter.services.tools
{
	import flash.events.Event;
	import flash.events.EventDispatcher;
	import flash.events.IOErrorEvent;
	import flash.net.URLLoader;
	import flash.net.URLRequest;
	import flash.net.URLRequestHeader;
	import flash.net.URLRequestMethod;
	import flash.net.URLVariables;
	
	import net.richardlord.smartcomponents.twitter.data.User;
	import net.richardlord.smartcomponents.twitter.events.ServiceEvent;
	
	import org.flaircode.oauth.IOAuth;
	
	public class TwitterRequest extends EventDispatcher
	{
		public static const FORMAT_TEXT : String = "text";
		public static const FORMAT_XML : String = "xml";
		public static const FORMAT_VARIABLES : String = "variables";
		public static const FORMAT_BINARY : String = "binary";
		
		[Inject]
		public var oauth : IOAuth;
		
		[Inject]
		public var user : User;
		
		public var url : String;
		public var sendData : * = null;
		public var sendMethod : String = URLRequestMethod.POST;
		public var loadDataFormat : String;
		public var loadData : *;
		
		private var _loader : URLLoader;
		
		public function TwitterRequest( loadDataFormat : String = FORMAT_TEXT )
		{
			this.loadDataFormat = loadDataFormat;
		}
		
		public function sendAndLoad() : void
		{
			if ( !url )
			{
				throw( new Error( "HTTPRequest requires a url to send to." ) );
			}
			
			_loader = new URLLoader();
			_loader.addEventListener( Event.COMPLETE, loadComplete );
			_loader.addEventListener( IOErrorEvent.IO_ERROR, loadFail );
			
			_loader.dataFormat = ( loadDataFormat == FORMAT_XML ? FORMAT_TEXT : loadDataFormat );
			
			var request : URLRequest;
			if ( user.token )
			{
				if ( sendData is URLVariables )
				{
					request = oauth.buildRequest( sendMethod, url, user.token, sendData );
					if ( sendMethod != URLRequestMethod.GET )
					{
						var split : Array = request.url.split( "?" );
						if ( split.length > 1 )
						{
							request.url = split[ 0 ];
							request.data = new URLVariables( split[ 1 ] );
						}
					}
				}
				else
				{
					request = oauth.buildRequest( sendMethod, url, user.token );
					if ( sendData )
					{
						request.data = sendData;
						if ( sendData is XML )
						{
							request.requestHeaders = [ new URLRequestHeader( "Content-Type", "application/xml" ) ];
						}
					}
				}
			}
			else
			{
				if( sendMethod == URLRequestMethod.GET && sendData is URLVariables )
				{
					request = new URLRequest( url + "?" + sendData.toString() );
					request.method = sendMethod;
				}
				else
				{
					request = new URLRequest( url );
					request.method = sendMethod;
					if ( sendData )
					{
						request.data = sendData;
						if ( sendData is XML )
						{
							request.requestHeaders = [ new URLRequestHeader( "Content-Type", "application/xml" ) ];
						}
					}
				}
			}
			_loader.load( request );
		}
		
		private function loadComplete( event : Event ) : void
		{
			if ( loadDataFormat == FORMAT_XML )
			{
				loadData = new XML( _loader.data );
			}
			else
			{
				loadData = _loader.data;
			}
			dispatchEvent( new ServiceEvent( ServiceEvent.REQUEST_COMPLETE, loadData ) );
		}
		
		private function loadFail( event : Event ) : void
		{
			dispatchEvent( new ServiceEvent( ServiceEvent.REQUEST_FAILED ) );
		}
	}
}